@@ -2,10 +2,10 @@ import tensorflow as tf
 from tensorflow import keras
 from tensorflow.python.types.core import TensorLike
 import numpy as np
-from copy import deepcopy
+import numpy.typing as npt
 from tqdm import tqdm, trange
 from collections import OrderedDict
-import typing
+import typing as T
 
 import logging
 
@@ -23,7 +23,7 @@ class LossTracker(OrderedDict):
             gives {'loss_a': [1, 0.7], 'loss_b': [2], 'loss_c': [1.2]}
     """
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key: T.Any, value: T.Any) -> None:
         try:
             # Assumes the key already exists
             # and the value is a list [oldest_value, another_old, ...]
@@ -34,21 +34,21 @@ class LossTracker(OrderedDict):
             # key -> [new_value]
             super(LossTracker, self).__setitem__(key, [value])
 
-    def to_numpy(self) -> np.array:
+    def to_numpy(self) -> npt.NDArray:
         """
         :return 2d vector of losses
         """
         _losses = np.array([np.array(v) for v in self.values() if isinstance(v, list)])
         return _losses
 
-    def labels(self) -> list:
+    def labels(self) -> T.List:
         """
         :return list of keys
         """
         return list(self.keys())
 
 
-class TimeGAN:
+class TimeGAN(keras.Model):
     """
     Time-series Generative Adversarial Networks (TimeGAN)
 
@@ -68,7 +68,8 @@ class TimeGAN:
         n_layers: int = 3,
         batch_size: int = 256,
         gamma: float = 1.0,
-    ):
+    ) -> None:
+        super().__init__()
         self.seq_len = seq_len
         self.hidden_dim = hidden_dim
         self.dim = n_features
@@ -128,19 +129,16 @@ class TimeGAN:
         # ----------------------------
         # Optimizers: call .compile() to set them
         # ----------------------------
-        DEFAULT_ADAM = keras.optimizers.Adam()
-        self.autoencoder_opt = deepcopy(DEFAULT_ADAM)
-        self.adversarialsup_opt = deepcopy(DEFAULT_ADAM)
-        self.generator_opt = deepcopy(DEFAULT_ADAM)
-        self.embedder_opt = deepcopy(DEFAULT_ADAM)
-        self.discriminator_opt = deepcopy(DEFAULT_ADAM)
+        self.autoencoder_opt = keras.optimizers.Adam()
+        self.adversarialsup_opt = keras.optimizers.Adam()
+        self.generator_opt = keras.optimizers.Adam()
+        self.embedder_opt = keras.optimizers.Adam()
+        self.discriminator_opt = keras.optimizers.Adam()
         # ----------------------------
         # Loss functions: call .compile() to set them
         # ----------------------------
-        DEFAULT_MSE = keras.losses.MeanSquaredError()
-        DEFAULT_BCE = keras.losses.BinaryCrossentropy()
-        self._mse = DEFAULT_MSE
-        self._bce = DEFAULT_BCE
+        self._mse = keras.losses.MeanSquaredError()
+        self._bce = keras.losses.BinaryCrossentropy()
 
         # --------------------------
         # All losses: will be populated in .fit()
@@ -154,14 +152,14 @@ class TimeGAN:
 
     def compile(
         self,
-        d_optimizer: keras.optimizers.Optimizer = keras.optimizers.Adam(),
-        g_optimizer: keras.optimizers.Optimizer = keras.optimizers.Adam(),
-        emb_optimizer: keras.optimizers.Optimizer = keras.optimizers.Adam(),
-        supgan_optimizer: keras.optimizers.Optimizer = keras.optimizers.Adam(),
-        ae_optimizer: keras.optimizers.Optimizer = keras.optimizers.Adam(),
+        d_optimizer: keras.optimizers.Optimizer = keras.optimizers.legacy.Adam(),
+        g_optimizer: keras.optimizers.Optimizer = keras.optimizers.legacy.Adam(),
+        emb_optimizer: keras.optimizers.Optimizer = keras.optimizers.legacy.Adam(),
+        supgan_optimizer: keras.optimizers.Optimizer = keras.optimizers.legacy.Adam(),
+        ae_optimizer: keras.optimizers.Optimizer = keras.optimizers.legacy.Adam(),
         emb_loss: keras.losses.Loss = keras.losses.MeanSquaredError(),
         clf_loss: keras.losses.Loss = keras.losses.BinaryCrossentropy(),
-    ):
+    ) -> None:
         """
         Assign optimizers and loss functions.
 
@@ -188,7 +186,7 @@ class TimeGAN:
         self._mse = emb_loss
         self._bce = clf_loss
 
-    def _define_timegan(self):
+    def _define_timegan(self) -> None:
         # --------------------------------
         # Data and Noise Inputs
         # --------------------------------
@@ -300,7 +298,7 @@ class TimeGAN:
     @tf.function
     def _train_generator(
         self, X: TensorLike, Z: TensorLike, optimizer: keras.optimizers.Optimizer
-    ) -> typing.Tuple[float, float, float, float, float]:
+    ) -> T.Tuple[float, float, float, float, float]:
         """
         3. Joint training (Generator training twice more than discriminator training): minimize G_loss
         """
@@ -343,7 +341,7 @@ class TimeGAN:
     @tf.function
     def _train_embedder(
         self, X: TensorLike, optimizer: keras.optimizers.Optimizer
-    ) -> typing.Tuple[float, float]:
+    ) -> T.Tuple[float, float]:
         """
         Train embedder during joint training: minimize E_loss
         """
@@ -431,7 +429,7 @@ class TimeGAN:
         while True:
             yield np.random.uniform(low=0, high=1, size=(self.seq_len, self.dim))
 
-    def get_noise_batch(self) -> typing.Iterator:
+    def get_noise_batch(self) -> T.Iterator:
         """
         Return an iterator of random noise vectors
         """
@@ -443,7 +441,7 @@ class TimeGAN:
             .repeat()
         )
 
-    def _get_data_batch(self, data, n_windows: int) -> typing.Iterator:
+    def _get_data_batch(self, data: TensorLike, n_windows: int) -> T.Iterator:
         """
         Return an iterator of shuffled input data
         """
@@ -457,10 +455,12 @@ class TimeGAN:
 
     def fit(
         self,
-        data: TensorLike,
+        data: T.Union[TensorLike, tf.data.Dataset],
         epochs: int,
-        checkpoints_interval: typing.Optional[int] = None,
-        generate_synthetic: tuple = (),
+        checkpoints_interval: T.Optional[int] = None,
+        generate_synthetic: T.Tuple = (),
+        *args,
+        **kwargs,
     ):
         """
         :param data: TensorLike, the training data
@@ -483,51 +483,56 @@ class TimeGAN:
             self._mse is None or self._bce is None
         ), "One of the loss functions is not defined. Please call .compile() to set them"
 
+        # take tf.data.Dataset | TensorLike
+        if isinstance(data, tf.data.Dataset):
+            batches = iter(data.repeat())
+        else:
+            batches = self._get_data_batch(data, n_windows=len(data))
+
         # Define the model
         self._define_timegan()
 
         # 1. Embedding network training
-        print("Start Embedding Network Training")
+        logger.info("Start Embedding Network Training")
 
         for epoch in tqdm(range(epochs), desc="Autoencoder - training"):
-            X_ = next(self._get_data_batch(data, n_windows=len(data)))
+            X_ = next(batches)
             step_e_loss_0 = self._train_autoencoder(X_, self.autoencoder_opt)
 
             # Checkpoint
             if checkpoints_interval is not None and epoch % checkpoints_interval == 0:
-                print(f"step: {epoch}/{epochs}, e_loss: {step_e_loss_0}")
+                logger.info(f"step: {epoch}/{epochs}, e_loss: {step_e_loss_0}")
             self.training_losses_history["autoencoder"] = float(step_e_loss_0)
 
-        print("Finished Embedding Network Training")
+        logger.info("Finished Embedding Network Training")
 
         # 2. Training only with supervised loss
-        print("Start Training with Supervised Loss Only")
+        logger.info("Start Training with Supervised Loss Only")
 
         # Adversarial Supervised network training
         for epoch in tqdm(range(epochs), desc="Adversarial Supervised - training"):
-            X_ = next(self._get_data_batch(data, n_windows=len(data)))
+            X_ = next(batches)
             step_g_loss_s = self._train_supervisor(X_, self.adversarialsup_opt)
 
             # Checkpoint
             if checkpoints_interval is not None and epoch % checkpoints_interval == 0:
-                print(
+                logger.info(
                     f"step: {epoch}/{epochs}, s_loss: {np.round(np.sqrt(step_g_loss_s), 4)}"
                 )
             self.training_losses_history["adversarial_supervised"] = float(
                 np.sqrt(step_g_loss_s)
             )
 
-        print("Finished Training with Supervised Loss Only")
+        logger.info("Finished Training with Supervised Loss Only")
 
         # 3. Joint Training
-        print("Start Joint Training")
+        logger.info("Start Joint Training")
 
         # GAN with embedding network training
         for epoch in tqdm(range(epochs), desc="GAN with embedding - training"):
-
             # Generator training (twice more than discriminator training)
             for kk in range(2):
-                X_ = next(self._get_data_batch(data, n_windows=len(data)))
+                X_ = next(batches)
                 Z_ = next(self.get_noise_batch())
                 # --------------------------
                 # Train the generator
@@ -545,16 +550,18 @@ class TimeGAN:
                 # --------------------------
                 _, step_e_loss_t0 = self._train_embedder(X_, self.embedder_opt)
 
-            X_ = next(self._get_data_batch(data, n_windows=len(data)))
+            X_ = next(batches)
             Z_ = next(self.get_noise_batch())
             step_d_loss = self._check_discriminator_loss(X_, Z_)
             if step_d_loss > 0.15:
-                print("Train Discriminator (discriminator does not work well yet)")
+                logger.info(
+                    "Train Discriminator (discriminator does not work well yet)"
+                )
                 step_d_loss = self._train_discriminator(X_, Z_, self.discriminator_opt)
 
             # Print multiple checkpoints
             if checkpoints_interval is not None and epoch % checkpoints_interval == 0:
-                print(
+                logger.info(
                     f"""step: {epoch}/{epochs},
                     d_loss: {np.round(step_d_loss, 4)},
                     g_loss_u: {np.round(step_g_loss_u, 4)},
@@ -577,7 +584,7 @@ class TimeGAN:
                 _sample = self.generate(n_samples=len(data))
                 self.synthetic_data_generated_in_training[epoch] = _sample
 
-        print("Finished Joint Training")
+        logger.info("Finished Joint Training")
         return
 
     def generate(self, n_samples: int) -> TensorLike:
